package com.halo.halojuhesdk.core.base.net;

import android.content.Context;
import android.text.TextUtils;

import com.halo.halojuhesdk.core.base.AbstractTask;
import com.halo.halojuhesdk.core.base.ReqType;
import com.halo.halojuhesdk.core.base.utils.LogUtils;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;

/**
 * Created by dream on 2016/11/10.
 */

public abstract class AbstractNetTask extends AbstractTask implements HlConnectionListener {
    // 是否为欢聚请求
    protected boolean isHjRequest = true;
    // 是否为实时请求
    protected boolean isHotTimeRequest = false;
    // 是否为Gzip请求
    protected boolean isGzipRequest = false;
    // 当前请求网络限制
    protected int netLevel = NetworkManager.HJ_ACCEPT_NET_LEVEL;
    /**
     * HTTP链接超时时间
     */
    private static final int HTTP_TIMEOUT_MS = 180000;
    /**
     * 请求获取数据超时时间
     */
    private static final int SOCKET_TIMEOUT_MS = 180000;
    /**
     * HTTP链接超时时间
     */
    private static final int HTTP_TIMEOUT = 100 * 1000;
    /**
     * 请求获取数据超时时间
     */
    private static final int SOCKET_TIMEOUT = 100 * 1000;

    /**
     * HTTP链接超时时间
     */
    private static final int HTTP_MIN_TIMEOUT_MS = 3000;
    /**
     * 请求获取数据超时时间
     */
    private static final int SOCKET_MIN_TIMEOUT_MS = 3000;
    private static final String TAG = "AbstractNetTask";

    /**
     * 没有网络连接
     */
    public static final int ERROR_NOT_CONNECT = -1;
    /**
     * 网络连接异常
     */
    public static final int ERROR_CONNECT_EXCEPTION = -2;
    /**
     * 未知网络类型
     */
    public static final int ERROR_UNKNOWN_NET_TYPE = -3;
    /**
     * 当前网络类型和任务支持的网络类型不匹配
     */
    public static final int ERROR_NET_TYPE_MISMATCH = -4;

    public static final int ERROR_FILE_NOT_FOUND = -5;

    public static final int ERROR_UNKNOWN_HOST = -6;

    public AbstractNetTask(Context context, boolean isNeed) {
        super(context, isNeed);
    }

    protected abstract String getURL();

    protected abstract void getEntity(OutputStream outputStream);

    protected abstract void onAddHeaders(HttpURLConnection connection);

    protected abstract ReqType getReqType();

    @Override
    protected boolean preExecute() {
        if (!NetworkManager.isNetworkConnected(mContext)) {
            onNetworkError(ERROR_NOT_CONNECT, "没有网络连接");
            return false;
        }
        if (NetworkManager.getNetworkType(mContext) > netLevel) {
            onNetworkError(ERROR_UNKNOWN_NET_TYPE, "允许的网络等级限制,netLevel : " + netLevel);
            return false;
        }
        return true;
    }

    @Override
    protected void onExcute() {
        String url = getURL();
        if (!TextUtils.isEmpty(url)) {
            try {
                HlConnectionManager hjConnectionManager = new HlConnectionManager(getReqType(), url);
                hjConnectionManager.setGzipRequest(isGzipRequest);
                if (isHotTimeRequest) {
                    hjConnectionManager.setTimeOut(HTTP_MIN_TIMEOUT_MS, SOCKET_MIN_TIMEOUT_MS);
                } else {
                    hjConnectionManager.setTimeOut(HTTP_TIMEOUT_MS, SOCKET_TIMEOUT_MS);
                }
                hjConnectionManager.openConnection(this);

            } catch (FileNotFoundException e) {
                LogUtils.e("出错了。");
                LogUtils.e( e);
                e.printStackTrace();
            } catch (Exception e) {
                LogUtils.e( e);
                e.printStackTrace();
            }
        }
    }

    private void onNetworkError(int errorCode, String errorMsg) {
        mListener.onError(errorCode, errorMsg);
        mExcuteSucc = false;
    }

    @Override
    public void onRecive(int code, String content) {
        mListener.finish(code, content);
    }

    public HttpURLConnection openConnection(String url) throws Exception {

        if (TextUtils.isEmpty(url)) {
            LogUtils.e( "url == null");
            return null;
        }

        URL Url = new URL(url);

        String str_http = url.substring(0, url.indexOf(":"));

        HttpURLConnection connection;
        if (str_http.equals("https")) {
            connection = getHttpsURLConnection(Url);
        } else {
            connection = getHttpURLConnection(Url);
        }
        return dealWithHttpConnection(connection);
    }

    private HttpsURLConnection getHttpsURLConnection(URL url) throws IOException, NoSuchAlgorithmException, KeyManagementException {
        HttpsURLConnection httpsURLConnection;
        httpsURLConnection = (HttpsURLConnection) url.openConnection();
        httpsURLConnection.setConnectTimeout(SOCKET_TIMEOUT);
        httpsURLConnection.setReadTimeout(HTTP_TIMEOUT);
        TrustManager[] tm = {new MyX509TrustManager()};
        SSLContext sslContext = SSLContext.getInstance("SSL");
        sslContext.init(null, tm, null);
        httpsURLConnection.setSSLSocketFactory(sslContext.getSocketFactory());
        httpsURLConnection.setHostnameVerifier(new TrustAnyHostnameVerifier());
        httpsURLConnection.setInstanceFollowRedirects(true);
        httpsURLConnection.setRequestProperty("Accept-Charset", "UTF-8");
        return httpsURLConnection;
    }

    private HttpURLConnection getHttpURLConnection(URL url) throws IOException {
        HttpURLConnection httpURLConnection;
        httpURLConnection = (HttpURLConnection) url.openConnection();
        httpURLConnection.setConnectTimeout(SOCKET_TIMEOUT);
        httpURLConnection.setReadTimeout(HTTP_TIMEOUT);
        httpURLConnection.setInstanceFollowRedirects(true);
        httpURLConnection.setRequestProperty("Accept-Charset", "UTF-8");
        return httpURLConnection;
    }

    private HttpURLConnection dealWithHttpConnection(HttpURLConnection connection) throws Exception {
        if (isGzipRequest) {
            connection.addRequestProperty("Content-Type", "text/html");
            connection.addRequestProperty("Accept-Encoding", "gzip,deflate");
        } else {
            connection.addRequestProperty("Accept-Encoding", "");
        }
        onAddHeaders(connection);
        connection.setRequestMethod("GET");
        return connection;
    }


}
