package com.halo.halojuhesdk.core.channel.applovin;

import android.app.Activity;
import android.util.Log;

import com.applovin.mediation.MaxAd;
import com.applovin.mediation.MaxAdListener;
import com.applovin.mediation.MaxError;
import com.applovin.mediation.ads.MaxInterstitialAd;
import com.halo.halojuhesdk.core.base.utils.CompareEcpmUtils;
import com.halo.halojuhesdk.core.channel.IInsertAd;
import com.halo.halojuhesdk.core.channel.IInsertProxyListener;
import com.halo.halojuhesdk.core.request.EcpmCompareBean;
import com.halo.halojuhesdk.core.request.InitDataProcessor;
import com.halo.halojuhesdk.core.request.SlotBean;
import com.halo.halojuhesdk.sdk.HlAdClient;

public class AppLovinInsert implements IInsertAd, MaxAdListener {
    private Activity mContext;
    private MaxInterstitialAd interstitialAd;
    private SlotBean mLovinBean;
    private IInsertProxyListener mInsertListener;
    private boolean isShow = true;
    private boolean isClick = false;
    public AppLovinInsert(Activity activity, SlotBean bean, IInsertProxyListener listener) {
        mContext = activity;
        mInsertListener = listener;
        mLovinBean = bean;
        Boolean isInitSuccess = HlAdClient.initSuccessMap.get("max");
        if (isInitSuccess == null || !isInitSuccess) {
            try {
                InitDataProcessor.initAppLovin(activity, bean.extendStr);
                HlAdClient.initSuccessMap.put("max", true);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        interstitialAd = new MaxInterstitialAd(bean.adSlotId, activity);
        interstitialAd.setListener(this);
    }


    @Override
    public void loadAd() {
        if (interstitialAd != null) {
            isShow = true;
            isClick = false;
            interstitialAd.loadAd();
        }
    }

    @Override
    public void showAd() {
        if (interstitialAd != null) {
            interstitialAd.showAd();
        }
    }

    @Override
    public void release() {

    }

    @Override
    public void onAdLoaded(MaxAd maxAd) {
        double ecpm = maxAd.getRevenue()*1000;
        mLovinBean.setEp(ecpm);
        EcpmCompareBean bean = CompareEcpmUtils.compareEcpm(mLovinBean, ecpm);
        mLovinBean.setOp(bean.getEcpm());
        if (bean.getResult()) {
            mInsertListener.onAdReady(mLovinBean, InitDataProcessor.SDK_MAX, bean.getEcpm());
        } else {
            mInsertListener.onAdError("applovin: 竞价失败", 102, InitDataProcessor.SDK_MAX, mLovinBean);
        }
    }

    @Override
    public void onAdDisplayed(MaxAd maxAd) {
        if (mInsertListener != null) {
            if (isShow) {
                isShow = false;
                mInsertListener.onDisplayAd(mLovinBean);
            }
        }
    }

    @Override
    public void onAdHidden(MaxAd maxAd) {
        if (mInsertListener != null) {
            mInsertListener.onCloseAd();
        }
    }

    @Override
    public void onAdClicked(MaxAd maxAd) {
        if (!isClick) {
            isClick = true;
            if (mInsertListener != null) {
                mInsertListener.onClickAd(mLovinBean);
            }
        }
    }

    @Override
    public void onAdLoadFailed(String s, MaxError maxError) {
        Log.e("11111","====onAdLoadFailed====" + maxError.getMessage());
        if (mInsertListener != null) {
            mInsertListener.onAdError("lovin:error===" + maxError.getMessage(), maxError.getCode(), InitDataProcessor.SDK_MAX, mLovinBean);
        }
    }

    @Override
    public void onAdDisplayFailed(MaxAd maxAd, MaxError maxError) {

    }
}
