package com.halo.halojuhesdk.sdk;

import android.app.Activity;
import android.text.TextUtils;
import android.view.ViewGroup;

import com.halo.halojuhesdk.core.base.utils.AnalyseSlotBeanUtils;
import com.halo.halojuhesdk.core.base.utils.LogUtils;
import com.halo.halojuhesdk.core.base.utils.Utils;
import com.halo.halojuhesdk.core.channel.ISplashAd;
import com.halo.halojuhesdk.core.channel.ISplashProxyListener;
import com.halo.halojuhesdk.core.channel.applovin.AppLovinReward;
import com.halo.halojuhesdk.core.channel.applovin.AppLovinSplash;
import com.halo.halojuhesdk.core.common.utils.Config;
import com.halo.halojuhesdk.core.common.utils.MyUtils;
import com.halo.halojuhesdk.core.request.HlStatisManager;
import com.halo.halojuhesdk.core.request.InitDataProcessor;
import com.halo.halojuhesdk.core.request.InitStatus;
import com.halo.halojuhesdk.core.request.RequestAdManager;
import com.halo.halojuhesdk.core.request.SlotBean;
import com.halo.halojuhesdk.core.request.UpRequestCallBack;
import com.halo.halojuhesdk.sdk.listener.HlSplashAdListener;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.UUID;

public class HlSplashAd {
    private Activity mContext;
    private HlSplashAdListener mListener;
    private Activity mActivity;
    private ViewGroup mAdContainer;
    private Map<String, Boolean> splashAdMap;
    private Map<String, SlotBean> splashPriceAdMap;
    private Map<String, Double> ecpmAdMap;
    private Map<String, String> adSubTypeMap;
    private int maxTime;
    private boolean isLoadAd;
    private boolean isAdClose;
    private JSONArray mPostJson;
    private boolean canJump;
    private boolean isDisplay;
    private int timeCount = 0;
    private Map<String, ISplashAd> splashViewMap;
    private String mCurrentFrom;
    private boolean isTimerFinish;
    private Timer timerStatus;
    double currentPrice = 0;
    private String mSlotId = "";
    private String mCurrentAdSlotId;
    private String mSplashReqId = UUID.randomUUID().toString();
    private TimerTask timerTask;
    private Timer timerLopper;
    private TimerTask timerLooperTask;

    public HlSplashAd(Activity context, String slotId, ViewGroup adContainer, HlSplashAdListener listener) {
        if (context == null) {
            mListener.onAdError("开屏广告context不能为空", InitStatus.INIT_FAIL_CODE);
            return;
        }
        MyUtils.init(context);
        mContext = context;
        maxTime = Config.AD_TIME_OUT;
        isLoadAd = false;
        isDisplay = false;
        canJump = false;
        isAdClose = false;
        mSlotId = slotId;
        mCurrentFrom = "";
        mCurrentAdSlotId = "";
        isTimerFinish = false;
        splashAdMap = new HashMap<>();
        splashPriceAdMap = new HashMap<>();
        ecpmAdMap = new HashMap<>();
        adSubTypeMap = new HashMap<>();
        mPostJson = new JSONArray();
        mListener = listener;
        mActivity = context;
        mAdContainer = adContainer;
        splashViewMap = new HashMap<>();
        onResume();
        loadAndShowSplashAd(slotId);
    }

    private void loadAndShowSplashAd(String slotId) {
        List<SlotBean> slotBeanList = RequestAdManager.requestChannel(slotId);
        if (slotBeanList == null || slotBeanList.size() == 0) {
            if (mListener != null) {
                mListener.onAdError("开屏初始化失败", InitStatus.INIT_FAIL_CODE);
                reportErrorMsg("开屏初始化失败", InitStatus.INIT_FAIL_CODE, "初始化", null);
            }
        } else {
            try {
                initSplash(slotBeanList);
            } catch (Exception e) {
                e.printStackTrace();
                mListener.onAdError("开屏广告加载失败:" + e, InitStatus.INIT_FAIL_CODE);
                reportErrorMsg("开屏初始化失败" + e, InitStatus.INIT_FAIL_CODE, "初始化", null);
            }
        }
    }

    private void initSplash(final List<SlotBean> baseBean) {
        final List<SlotBean> currentList = new ArrayList<>();
        for (int i = 0; i < baseBean.size(); i++) {
            final SlotBean bean = baseBean.get(i);
            if (bean == null) {
                continue;
            }
            String adFrom = bean.getAdFrom();
            if (adFrom.equals(InitDataProcessor.SDK_MAX) && HlAdClient.mTypeList.contains(Config.JuHeAdType.MAX)) {
                AppLovinSplash appLovinsplash = new AppLovinSplash(mContext, bean, proxyListener);
                currentList.add(bean);
                splashViewMap.put(bean.adSlotId, appLovinsplash);
            }
        }
        if (currentList.isEmpty()) {
            mListener.onAdError("开屏初始化失败, 无相应广告配置", InitStatus.INIT_FAIL_CODE);
            reportErrorMsg("开屏初始化失败, 无相应广告配置", InitStatus.INIT_FAIL_CODE, "初始化", null);
            return;
        }
        timerLopper = new Timer();
        timerLooperTask = new TimerTask() {
            @Override
            public void run() {
                maxTime = maxTime - 100;
                if (splashAdMap.size() == currentList.size()) {
                    timerLopper.cancel();
                    formatSplashResult(currentList);
                } else if (maxTime <= 0) {
                    timerLopper.cancel();
                    if (!splashAdMap.isEmpty()) {
                        if (isLoadAd) return;
                        formatSplashResult(currentList);
                    } else {
                        for (SlotBean bean : currentList) {
                            analyseBeanObj(bean, false, false, 0);
                        }
                        HlStatisManager.getInstance().reportStat(mPostJson.toString(), mSplashReqId);
                        onAdError("超时没有加载到开屏广告");
                    }
                }
            }
        };
        timerLopper.schedule(timerLooperTask, 0, 100);
    }

    private void formatSplashResult(List<SlotBean> currentList) {
        String currentAdSlotId = "";
        for (Map.Entry<String, Double> entry : ecpmAdMap.entrySet()) {
            String key = entry.getKey();
            double value = entry.getValue();
            if (value >= currentPrice) {
                currentAdSlotId = key;
                currentPrice = value;
            }
        }
        if ((currentPrice == 0 || currentPrice == 1) && !currentList.isEmpty()) {
            for (SlotBean bean : currentList) {
                Boolean aBoolean = splashAdMap.get(bean.adSlotId);
                if (aBoolean != null && aBoolean && ecpmAdMap.containsKey(bean.adSlotId)) {
                    currentAdSlotId = bean.adSlotId;
                    break;
                }
            }
        }
//        LogUtils.e("=====splashsize====" + ecpmAdMap.size() + "===currentAdSlotId===" + currentAdSlotId + "====currentPrice====" + currentPrice);
        for (final SlotBean bean : currentList) {
            final String adSlotId = bean.adSlotId;
            Boolean aBoolean = splashAdMap.get(adSlotId);
            if (aBoolean != null && aBoolean) {
                Double ecpmNum = ecpmAdMap.get(adSlotId);
                if (ecpmNum == null) {
                    ecpmNum = 0d;
                }
                bean.setOp(ecpmNum);
                if (!isLoadAd && !TextUtils.isEmpty(currentAdSlotId) && currentAdSlotId.equals(adSlotId)) {
                    if (splashPriceAdMap != null && splashPriceAdMap.containsKey(adSlotId)) {
                        analyseBeanObj(bean, true, false, ecpmNum);
                        continue;
                    }
                    isLoadAd = true;
                    mCurrentFrom = bean.adFrom;
                    mCurrentAdSlotId = bean.adSlotId;
                    LogUtils.e("====mSplashShowFrom====" + bean.adFrom);
                    mActivity.runOnUiThread(() -> {
                        String adType = adSubTypeMap.get(bean.adSlotId);
                        mListener.onReadyAd(adType);
                    });
                    analyseBeanObj(bean, true, true, ecpmNum);
                } else {
                    analyseBeanObj(bean, true, false, ecpmNum);
                }
            } else {
                analyseBeanObj(bean, false, false, 0);
            }
        }
        HlStatisManager.getInstance().reportStat(mPostJson.toString(), mSplashReqId);
        if (!isLoadAd) {
            onAdError("没有加载到开屏广告源");
        }
    }

    private void analyseBeanObj(SlotBean mBean, boolean isQuota, boolean isUse, double op) {
        try {
            JSONObject parameJson = AnalyseSlotBeanUtils.analyseBean(mBean, isQuota, isUse, op, mSplashReqId);
            mPostJson.put(parameJson);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void loopSplashStatus() {
        timerStatus = new Timer();
        timerTask = new TimerTask() {
            @Override
            public void run() {
                if (timeCount == 6 && isLoadAd && !isDisplay) {
                    timerStatus.cancel();
                    LogUtils.e("====loopCloseStatus====");
                    isAdClose = true;
                    mActivity.runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            mListener.onCloseAd();
                        }
                    });
                }
                timeCount++;
            }
        };
        timerStatus.schedule(timerTask, 0, 1000);
    }

    private void onAdError(final String errorMsg) {
        reportErrorMsg(errorMsg, 100, "广告源", null);
        mActivity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                LogUtils.e("====error====" + errorMsg);
                mListener.onAdError(errorMsg, 100);
            }
        });
    }

    public void showAd() {
        loopSplashStatus();
        if (!TextUtils.isEmpty(mCurrentAdSlotId)) {
            ISplashAd iSplashAd = splashViewMap.get(mCurrentAdSlotId);
            if (splashViewMap != null && iSplashAd != null) {
                iSplashAd.LoadAndShowAd();
            } else {
                reportErrorMsg("开屏广告load失败:", InitStatus.INIT_FAIL_CODE, "广告源", null);
                mActivity.runOnUiThread(() -> mListener.onCloseAd());
            }
        } else {
            reportErrorMsg("开屏广告load失败:", InitStatus.INIT_FAIL_CODE, "广告源", null);
            mActivity.runOnUiThread(() -> mListener.onCloseAd());
        }
    }

    private void loadSplashAd(SlotBean bean) {
        ISplashAd iSplashAd = splashViewMap.get(bean.adSlotId);
        if (splashViewMap != null && iSplashAd != null) {
            iSplashAd.LoadAndShowAd();
        } else {
            mActivity.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    mListener.onAdError("开屏广告load失败:", InitStatus.INIT_FAIL_CODE);
                }
            });
        }
    }

    public double getECPM() {
        return currentPrice;
    }

    public Map<String, Double> getEcpmMap() {
        return ecpmAdMap;
    }

    private final ISplashProxyListener proxyListener = new ISplashProxyListener() {
        @Override
        public void onReadyAd(SlotBean bean, String adType, double ecpm, String adTypeStr) {
            LogUtils.e("====onReadyAd====" + adType);
            splashAdMap.put(bean.adSlotId, true);
            ecpmAdMap.put(bean.adSlotId, ecpm);
            adSubTypeMap.put(bean.adSlotId, adTypeStr);
        }

        @Override
        public void onDisplayAd(final SlotBean bean, String adTypeStr) {
            isDisplay = true;
            if (isAdClose) return;
            LogUtils.e("====HlonDisplayAd====" + bean.adFrom);
            if (bean.adFrom.contains("sdk")) {
                Double ecpmNum = ecpmAdMap.get(bean.adSlotId);
                if (ecpmNum == null) {
                    ecpmNum = 0d;
                }
                bean.setOp(ecpmNum);
                HlStatisManager.getInstance().reportStaticCallBack(bean, HlStatisManager.STEPNAME_REPORT, HlStatisManager.STEPNAME_REPORT_IMP, mSplashReqId, new UpRequestCallBack() {
                    @Override
                    public void onSuccess() {
                        mActivity.runOnUiThread(() -> {
                            if (mListener != null) {
                                mListener.onDisplayAd(adTypeStr);
                            }
                        });
                    }

                    @Override
                    public void onError() {
                    }
                });
            } else {
                mActivity.runOnUiThread(() -> {
                    if (mListener != null) {
                        mListener.onDisplayAd(adTypeStr);
                    }
                });
            }
        }

        @Override
        public void onClickAd(final SlotBean bean, String adTypeStr) {
            closLoopTask();
            if (mListener != null) {
                isTimerFinish = true;
                if (bean.adFrom.contains("sdk")) {
                    HlStatisManager.getInstance().reportStaticCallBack(bean, HlStatisManager.STEPNAME_REPORT, HlStatisManager.STEPNAME_REPORT_CLICK, mSplashReqId, new UpRequestCallBack() {
                        @Override
                        public void onSuccess() {
                            mActivity.runOnUiThread(() -> mListener.onClickAd(adTypeStr));
                        }

                        @Override
                        public void onError() {
                        }
                    });
                } else {
                    mActivity.runOnUiThread(() -> mListener.onClickAd(adTypeStr));
                }
            }
        }

        @Override
        public void onCloseAd() {
            isAdClose = true;
            if (mListener != null) {
                isTimerFinish = true;
                if (!canJump) {
                    jumpDismiss();
                }
            }
        }

        @Override
        public void onAdSkip() {
            closLoopTask();
            if (mListener != null) {
                mActivity.runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        mListener.onAdSkip();
                    }
                });
            }
        }

        @Override
        public void onAdError(String msg, int code, String type, SlotBean bean) {
            LogUtils.e("====errorMsg===" + msg + "====code===" + code + "===onAdError====" + type + "===adSoltId===" + bean.adSlotId);
            if (code == 102) {
                splashPriceAdMap.put(bean.adSlotId, bean);
                splashAdMap.put(bean.adSlotId, true);
            } else {
                splashAdMap.put(bean.adSlotId, false);
            }
            reportErrorMsg(msg, code, type, bean);
        }
    };

    private void reportErrorMsg(String msg, int code, String type, SlotBean bean) {
        if (bean == null) {
            bean = new SlotBean();
            bean.setAppId(Config.mAppId);
            bean.setSlotId(mSlotId);
        }
        String errorMsg = type + ": errorTime==" + Utils.formatCurrentTime() + "==errorMsg:" + msg + "==errorCode==" + code;
        HlStatisManager.getInstance().reportStaticError(bean, HlStatisManager.STEPNAME_ERROR, "", mSplashReqId, errorMsg);
    }

    public void onPause() {
        canJump = true;
    }

    public void onResume() {
        if (canJump && isTimerFinish) {
            jumpDismiss();
        }
        canJump = false;
    }

    public void onDestroy() {
        closLoopTask();
    }

    private void jumpDismiss() {
        isTimerFinish = false;
        closLoopTask();
        mListener.onCloseAd();
    }

    private void closLoopTask() {
        if (timerStatus != null) {
            timerStatus.cancel();
            timerStatus = null;
        }
        if (timerTask != null) {
            timerTask.cancel();
            timerTask = null;
        }
        if (timerLopper != null) {
            timerLopper.cancel();
            timerLopper = null;
        }
        if (timerLooperTask != null) {
            timerLooperTask.cancel();
            timerLooperTask = null;
        }
    }
}
