package com.halomobi.download;

import static com.halomobi.common.ConstantPool.A_SDK;
import static com.halomobi.download.bean.DownloadItem.STATE_CANCEL;
import static com.halomobi.download.bean.DownloadItem.STATE_DOWNLOADING;
import static com.halomobi.download.bean.DownloadItem.STATE_FINISH;
import static com.halomobi.download.bean.DownloadItem.STATE_INSTALLED;
import static com.halomobi.download.bean.DownloadItem.STATE_INTERRUPT;
import static com.halomobi.download.bean.DownloadItem.STATE_PAUSE;
import static com.halomobi.download.bean.DownloadItem.STATE_WAITING;
import static com.halomobi.download.database.DownloadDBHelper.CLICK_TRACKER;
import static com.halomobi.download.database.DownloadDBHelper.DEEP_LINK_TRACKER;
import static com.halomobi.download.database.DownloadDBHelper.DOWNLOADED_TRACKER;
import static com.halomobi.download.database.DownloadDBHelper.DOWNLOAD_START_TRACKER;
import static com.halomobi.download.database.DownloadDBHelper.INSTALLED_TRACKER;
import static com.halomobi.download.database.DownloadDBHelper.INSTALL_START_TRACKER;
import static com.halomobi.download.database.DownloadDBHelper.OPEN_TRACKER;
import static com.halomobi.download.receiver.DownloadReceiver.ACTION_CANCEL_DOWNLOAD;
import static com.halomobi.download.receiver.DownloadReceiver.ACTION_DOWNLOAD_CONTINUE;
import static com.halomobi.download.receiver.DownloadReceiver.ACTION_DOWNLOAD_INSTALL;
import static com.halomobi.download.receiver.DownloadReceiver.ACTION_DOWNLOAD_OPEN;
import static com.halomobi.download.receiver.DownloadReceiver.ACTION_DOWNLOAD_PAUSE;
import static com.halomobi.download.receiver.DownloadReceiver.DOWNLOAD_URL;
import static com.halomobi.download.receiver.DownloadReceiver.NOTIFICATION_ID;
import static com.halomobi.download.receiver.DownloadReceiver.PACKAGE_NAME;

import android.app.Notification;
import android.app.NotificationChannel;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.ActivityNotFoundException;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.content.res.AssetManager;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.drawable.Drawable;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.Uri;
import android.os.Build;
import android.text.TextUtils;

import androidx.core.app.NotificationCompat;
import androidx.core.content.FileProvider;


import com.halomobi.common.ConstantPool;
import com.halomobi.download.bean.DownloadItem;
import com.halomobi.download.database.DownloadDBManager;
import com.halomobi.download.listener.DownloadListener;
import com.halomobi.download.listener.DownloadStateListener;
import com.halomobi.network.report.track.ReportTrackerManager;
import com.halomobi.network.request.ad.bean.Ad;
import com.halomobi.network.schedule.ThreadManager;
import com.halomobi.utils.FileUtils;
import com.halomobi.utils.KeyUtil;
import com.halomobi.utils.LogUtils;
import com.halomobi.utils.NetworkUtils;
import com.halomobi.utils.SDKInfo;
import com.halomobi.utils.Utils;

import java.io.File;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.net.URISyntaxException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

public class DownLoadManager {
    private static DownLoadManager manager;
    private static Context mContext;
    private ArrayList<DownloadItem> mItems = new ArrayList<>();

    private DecimalFormat fileDecimalFormat = new DecimalFormat("#0.##");

    public static boolean sIsFromSDK = false;
    private String channel_id = "ssp_notification_id";
    private String channel_name = "ssp_notification_name";
    private Ad mAdInfo;

    private DownLoadManager() {
//        modifyFrom();
        mItems = DownloadDBManager.getInstance(mContext).query();
        init(false);
    }

    public static synchronized DownLoadManager getInstance(Context context) {
        mContext = context;
        if (manager == null) {
            manager = new DownLoadManager();
        }
        return manager;
    }

    /**
     * 查询数据库，继续执行未完成的下载任务（自动删除过期任务）
     */
    public void init(boolean isNetChange) {
        ConnectivityManager connectivityManager = (ConnectivityManager) mContext.getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo info = connectivityManager.getActiveNetworkInfo();

//        for (DownloadItem item : mItems) {
//            switch (item.getCurrentState()) {
//                case STATE_FINISH:
//                    LogUtils.e("下载完成");
//                    File file = new File(item.getSavePath());
//                    if (file.exists() && file.length() == item.getCurrentFileSize()) {
////                        setNotificationBuilder(item);
//                    } else if (info != null && info.isAvailable() && info.getType() == ConnectivityManager.TYPE_WIFI && info.isConnected()) {
//                        download(item);
//                    } else {
//                        item.setCurrentState(STATE_INTERRUPT);
//                        setNotificationBuilder(item);
//                    }
//                    break;
//                case STATE_INSTALLED:
//                    if (isAppInstalled(item.getPackageName(mContext))) {
//                        setNotificationBuilder(item);
//                    }
//                    break;
//                case STATE_WAITING:
//                case STATE_DOWNLOADING:
//                    if (isNetChange) {
//                        break;
//                    }
//                default:
//                    if (info != null && info.isAvailable() && info.getType() == ConnectivityManager.TYPE_WIFI && info.isConnected()) {
//                        download(item);
//                    } else {
//                        item.setCurrentState(STATE_INTERRUPT);
//                        setNotificationBuilder(item);
//                    }
//                    break;
//            }
//        }
    }

    /**
     * 下载任务并且展示Notification
     *
     * @param item 下载任务对象
     */
    public void downloadWithShow(final DownloadItem item, final Ad adInfo) {
        mAdInfo = adInfo;
        ThreadManager.getShortPool().execute(new Runnable() {
            @Override
            public void run() {
                asyncDownload(item, adInfo);
            }
        });
    }

    private synchronized void asyncDownload(DownloadItem item, Ad adInfo) {
        DownloadStateListener listener = item.getListener();
        if (TextUtils.isEmpty(item.getDownLoadUrl())) {
            LogUtils.w("downLoadUrl is Empty");
            return;
        }

        String path = FileUtils.getDownloadDir();
        String apkUrl =  KeyUtil.getMD5(item.getDownLoadUrl()) + ".apk";
        item.setSavePath(path + apkUrl);

        if (!FileUtils.createDirs(path)) {
            LogUtils.e("创建目录失败，中断下载任务");
            return;
        }

        int position = mItems.indexOf(item);

        if (position != -1) {
            DownloadItem info = mItems.get(position);
            //判断条件:1. 状态是安装完成状态  包名不为空  真实没有安装上
            if (STATE_INSTALLED == info.getCurrentState() && !TextUtils.isEmpty(info.getPackageName(mContext)) && !isAppInstalled(info.getPackageName(mContext))) {
                mItems.remove(info);
                mItems.add(item);
            } else {
                info.setClickTracker(item.getClickTracker());
                info.setDownloadStartTrackerTracker(item.getDownloadStartTracker());
                info.setDownloadedTracker(item.getDownloadedTracker());
                info.setInstallStartTracker(item.getDownloadStartTracker());
                info.setInstalledTracker(item.getInstalledTracker());
                info.setOpenTracker(item.getOpenTracker());
                info.setDeepLinkTracker(item.getDeepLinkTracker());
                info.setDeepLink(item.getDeepLink());
                info.setNetType(item.getNetType());
                item = info;
            }
        } else {
            mItems.add(item);
        }

        try {
            DownloadDBManager.getInstance(mContext).add(item);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (item.getListener() == null && listener != null) {
            item.setListener(listener);
        }
        start(item, adInfo);
    }

    /**
     * 包含已下载完成任务的notification通知和未完成任务继续下载
     *
     * @param item 下载任务对象
     */
    private void start(DownloadItem item, Ad adInfo) {
        switch (item.getCurrentState()) {
            case STATE_FINISH:
                File file = new File(item.getSavePath());
                if (file.exists() && file.length() == item.getCurrentFileSize()) {
//                    Utils.showToastSafe("下载完成");
                    setNotificationBuilder(item);
                    if (item.getListener() != null) {
                        item.getListener().onDownloaded(item, adInfo);
                    }
                    installApp(item);
                } else {
                    download(item);
                }
                break;
            case STATE_DOWNLOADING:
            case STATE_WAITING:
//                Utils.showToastSafe("正在下载");
                break;
            case STATE_INSTALLED:
                setNotificationBuilder(item);
                if (isAppInstalled(item.getPackageName(mContext))) {
                    //启动App
                    openApp(item, adInfo);
                }
                break;
            default:
                download(item);
                break;
        }
    }

    /**
     * 执行下载任务，包括新下载和断点续传
     *
     * @param item 下载任务对象
     */
    public void download(final DownloadItem item) {

        if (item == null || !NetworkUtils.isNetworkConnected(mContext)) {
            return;
        }
        item.setCurrentState(STATE_WAITING);

//        Utils.showToastSafe("开始下载");

        setNotificationBuilder(item);

        DownloadProcessor processor = new DownloadProcessor(item);
        processor.setListener(new DownloadListener() {
            @Override
            public void onStart(DownloadItem hjDownItem) {

                //文件大小 eTag 保存到数据库。
                DownloadDBManager.getInstance(mContext).update(hjDownItem);

                setNotificationBuilder(hjDownItem);

                //发送监播
                onDownloadState(hjDownItem, mAdInfo, DOWNLOAD_START_TRACKER);

                if (item.getListener() != null) {
                    item.getListener().onDownloadStart(hjDownItem, mAdInfo);
                }

            }

            @Override
            public void onProcessing(DownloadItem hjDownItem) {
                setNotificationBuilder(hjDownItem);
            }

            @Override
            public void onFinish(DownloadItem hjDownItem) {
                //下载完成后根据下载下来的apk包修改通知栏下载任务名称
                setApkName(hjDownItem);

//                if (isAppInstalled(hjDownItem.getPackageName(mContext))) {
//                    //启动App
//                    openApp(hjDownItem);
//                    hjDownItem.setCurrentState(STATE_INSTALLED);
//                } else {
                installApp(hjDownItem);
//                }
                setNotificationBuilder(hjDownItem);
                //存数据库，改变下载状态
                DownloadDBManager.getInstance(mContext).update(hjDownItem);
                //发送监播
                onDownloadState(hjDownItem, mAdInfo, DOWNLOADED_TRACKER);

                if (item.getListener() != null) {
                    item.getListener().onDownloaded(hjDownItem, mAdInfo);
                }
            }

            @Override
            public void onStop(DownloadItem hjDownItem) {
                setNotificationBuilder(hjDownItem);
                DownloadDBManager.getInstance(mContext).update(hjDownItem);
            }

            @Override
            public void onError(DownloadItem hjDownItem, int errorCode, String errorMsg) {

                setNotificationBuilder(hjDownItem);

                if (item.getListener() != null) {
                    item.getListener().onDownloadError(hjDownItem, errorCode, errorMsg);
                }
            }

            private void setApkName(DownloadItem downloadItem) {
                try {
                    PackageManager pm = mContext.getPackageManager();
                    PackageInfo info = pm.getPackageArchiveInfo(downloadItem.getSavePath(),
                            PackageManager.GET_ACTIVITIES);
                    Resources pRes = mContext.getResources();
                    if (info != null) {
                        CharSequence appName = null;
                        Resources res = new Resources(createAssetManager(downloadItem.getSavePath()), pRes.getDisplayMetrics(), pRes.getConfiguration());
                        ApplicationInfo appInfo = info.applicationInfo;
                        if (appInfo.labelRes != 0) {
                            try {
                                appName = res.getText(appInfo.labelRes);
                            } catch (Resources.NotFoundException e) {
                            }
                        }
                        if (appName == null) {
                            appName = (appInfo.nonLocalizedLabel != null) ? appInfo.nonLocalizedLabel : appInfo.packageName;
                        }
                        Drawable icon = null;
                        Bitmap bitmap = null;
                        if (appInfo.icon != 0) {
                            try {
                                icon = res.getDrawable(appInfo.icon);
                                bitmap = Utils.Drawable2BitMap(icon);
                            } catch (Resources.NotFoundException e) {
                            }
                        }
                        if (icon == null) {
                            icon = mContext.getPackageManager().getDefaultActivityIcon();
                            bitmap = Utils.Drawable2BitMap(icon);
                        }
                        if (bitmap != null) {
                            downloadItem.setIcon(bitmap);
                        }
                        downloadItem.setName(appName.toString());
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }

            private AssetManager createAssetManager(String apkPath) {
                try {
                    AssetManager assetManager = AssetManager.class.newInstance();
                    AssetManager.class.getDeclaredMethod("addAssetPath", String.class).invoke(
                            assetManager, apkPath);
                    return assetManager;
                } catch (Throwable th) {
                    th.printStackTrace();
                }
                return null;
            }
        });
        processor.process();
    }

    /**
     * 通过下载地址获取下载任务
     *
     * @param downloadUrl 下载地址
     * @return 下载任务对象
     */
    public DownloadItem getDownloadItem(String downloadUrl) {
        if (TextUtils.isEmpty(downloadUrl)) {
            return null;
        }
        for (DownloadItem item : mItems) {
            if (downloadUrl.equals(item.getDownLoadUrl())) {
                return item;
            }
        }

        return null;
    }

    /**
     * 移除Notification
     *
     * @param id Notification Id
     */
    private void removeNotification(int id) {
        NotificationManager manager = (NotificationManager)
                Utils.getContext().getSystemService(Context.NOTIFICATION_SERVICE);
        manager.cancel(id);
    }

    /**
     * 更新notification
     *
     * @param info 下载任务对象
     */
    private synchronized void setNotificationBuilder(final DownloadItem info) {

        if (info == null) {
            return;
        }

        try {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
                Intent contentIntent = new Intent();
                Intent deleteIntent = new Intent();
                String ticker;
                String contentTitle;
                int smallIconResId = android.R.drawable.stat_sys_download_done;
                int progressMax = 0;
                int progress = Integer.parseInt(fileDecimalFormat.format((int) (100 * ((double) info.getLocalFileSize() / info.getCurrentFileSize()))));
                String progressStr = "";
                switch (info.getCurrentState()) {
                    case STATE_DOWNLOADING:
                        contentTitle = "点击暂停下载";
                        ticker = "正在下载";
                        smallIconResId = android.R.drawable.stat_sys_download;
                        progressMax = 100;
                        progressStr = (progress > 100 ? 100 : progress < 0 ? 0 : progress) + "%";
                        contentIntent.setAction(ACTION_DOWNLOAD_PAUSE);
                        deleteIntent.setAction(ACTION_CANCEL_DOWNLOAD);
                        break;
                    case STATE_PAUSE:
                        contentTitle = "点击继续下载";
                        ticker = "暂停下载";
                        progressMax = 100;
                        progressStr = (progress > 100 ? 100 : progress < 0 ? 0 : progress) + "%";
                        contentIntent.setAction(ACTION_DOWNLOAD_CONTINUE);
                        break;
                    case STATE_FINISH:
                        contentTitle = "点击安装";
                        ticker = "下载完成";
                        contentIntent.setAction(ACTION_DOWNLOAD_INSTALL);
                        break;
                    case STATE_WAITING:
                        ticker = contentTitle = "等待下载中";
                        progressMax = 100;
                        progressStr = (progress > 100 ? 100 : progress < 0 ? 0 : progress) + "%";
                        contentIntent.setAction(ACTION_DOWNLOAD_PAUSE);
                        break;
                    case STATE_INSTALLED:
                        contentTitle = "点击打开应用";
                        ticker = "应用安装完成";
                        contentIntent.setAction(ACTION_DOWNLOAD_OPEN);
                        contentIntent.putExtra(PACKAGE_NAME, info.getPackageName(mContext));
                        contentIntent.putExtra(NOTIFICATION_ID, info.getDownLoadUrl().hashCode());
                        break;
                    case STATE_INTERRUPT:
                        contentTitle = "点击开始下载";
                        ticker = "准备开始下载";
                        contentIntent.setAction(ACTION_DOWNLOAD_CONTINUE);
                        break;
                    case STATE_CANCEL:
                        return;
                    default:
                        return;
                }

                contentIntent.putExtra(DOWNLOAD_URL, info.getDownLoadUrl());
                deleteIntent.putExtra(DOWNLOAD_URL, info.getDownLoadUrl());

                PendingIntent contentPendingIntent = PendingIntent.getBroadcast(mContext,
                        info.getDownLoadUrl().hashCode(), contentIntent, PendingIntent.FLAG_UPDATE_CURRENT);
                PendingIntent deletePendingIntent = PendingIntent.getBroadcast(mContext,
                        info.getDownLoadUrl().hashCode(), deleteIntent, PendingIntent.FLAG_UPDATE_CURRENT);

                if (info.getBuilder() == null) {
                    info.setBuilder(new NotificationCompat.Builder(mContext));
                    info.getBuilder().setTicker(ticker).setSmallIcon(smallIconResId)
                            .setWhen(System.currentTimeMillis())
                            .setContentTitle(info.getName())
                            .setContentText(contentTitle)
                            .setContentInfo(progressStr)
                            .setDefaults(Notification.FLAG_NO_CLEAR)
                            .setProgress(progressMax, progress, false)
                            //                    .setOngoing(true)
                            .setContentIntent(contentPendingIntent).setDeleteIntent(deletePendingIntent);
                    if (info.getIcon() != null) {
                        info.getBuilder().setLargeIcon(info.getIcon());
                    }
                } else {
                    info.getBuilder().setProgress(progressMax, progress, false)
                            .setSmallIcon(smallIconResId)
                            .setTicker(ticker).setContentText(contentTitle).setContentIntent(contentPendingIntent)
                            .setDeleteIntent(deletePendingIntent).setDefaults(Notification.FLAG_NO_CLEAR)
                            .setContentInfo(progressStr).setContentTitle(info.getName());
                    if (info.getIcon() != null) {
                        info.getBuilder().setLargeIcon(info.getIcon());
                    }
                }
                // 设置通知栏参数

                //显示通知栏
                NotificationManager manager = (NotificationManager)
                        mContext.getSystemService(Context.NOTIFICATION_SERVICE);
                if (Build.VERSION.SDK_INT >= 26) {
                    NotificationChannel notificationChannel = new NotificationChannel(channel_id, channel_name, NotificationManager.IMPORTANCE_DEFAULT);
                    notificationChannel.setSound(null, null);
                    notificationChannel.enableVibration(false);//震动不可用
                    manager.createNotificationChannel(notificationChannel);
                    info.getBuilder().setChannelId(channel_id);
                } else {
                    info.getBuilder().setSound(null);
                    info.getBuilder().setVibrate(new long[]{0});
                }
                Notification notification = info.getBuilder().build();
                manager.notify(info.getDownLoadUrl().hashCode(), notification);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 应用是否已经安装
     *
     * @return boolean    true 安装    false 未安装
     */
    public boolean isAppInstalled(String packageName) {
        if (TextUtils.isEmpty(packageName)) {
            return false;
        }
        List<ApplicationInfo> packages = mContext
                .getPackageManager().getInstalledApplications(0);
        for (ApplicationInfo packageInfo : packages) {
            if (packageInfo.packageName.equals(packageName)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 暂停下载
     *
     * @param item 下载任务对象
     */
    public void pauseDownload(DownloadItem item) {
        if (item == null) {
            return;
        }
        item.setCurrentState(STATE_PAUSE);
    }

    /**
     * 调起安装界面
     *
     * @param item 下载任务对象
     */
    public void installApp(DownloadItem item) {

        LogUtils.d("");
        try {

            if (!new File(item.getSavePath()).exists()) {
                download(item);
                return;
            }

            sIsFromSDK = true;

            //发送监播
            onDownloadState(item, mAdInfo, INSTALL_START_TRACKER);

            // 自动调起安装界面
            Intent intent = new Intent(Intent.ACTION_VIEW);
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            Uri data;
            File file = new File(item.getSavePath());
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                data = FileProvider.getUriForFile(mContext, mContext.getPackageName() + ".fileprovider", file);
                intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            } else {
                data = Uri.fromFile(file);
            }
            intent.setDataAndType(data, "application/vnd.android.package-archive");
            mContext.startActivity(intent);
        } catch (Exception e) {
            LogUtils.e("自动调起安装界面失败");
            e.printStackTrace();
        }
    }

    /**
     * 通过包名获取下载任务
     *
     * @param packName 包名
     * @return 下载任务
     */
    private DownloadItem getDownloadItemByPackName(String packName) {
        for (DownloadItem item : mItems) {
            if (packName.equals(item.getPackageName(mContext))) {
                return item;
            }
        }

        return null;
    }

    /**
     * 应用安装完成
     */
    public void installedApp(String packageName) {
        if (TextUtils.isEmpty(packageName)) {
            return;
        }

        DownloadItem item = getDownloadItemByPackName(packageName);

        if (item == null) {
            try {
                DownloadDBManager.getInstance(mContext).recordInstallFailedInfo(getProgramNameByPackageName(packageName), packageName);
            } catch (Exception e) {
                e.printStackTrace();
            }
            LogUtils.d(String.format("没有找到AppName : %s 的下载任务", getProgramNameByPackageName(packageName)));
            return;
        }

        item.setCurrentState(STATE_INSTALLED);

        DownLoadManager.getInstance(mContext).setNotificationBuilder(item);

        //发送监播
        onDownloadState(item, mAdInfo, INSTALLED_TRACKER);
        if (item.getListener() != null) {
            item.getListener().onInstalled(item, mAdInfo);
        }

        String deepLink = item.getDeepLink();
        if (!TextUtils.isEmpty(deepLink)) {
            try {
                Intent deepLinkIntent = Intent.parseUri(deepLink, Intent.URI_INTENT_SCHEME);
                deepLinkIntent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                ResolveInfo resolveInfo = mContext.getPackageManager().resolveActivity(deepLinkIntent, PackageManager.MATCH_DEFAULT_ONLY);
                if (resolveInfo != null) {
                    mContext.startActivity(deepLinkIntent);
                    LogUtils.d("DeepLink 2 跳转成功   ：  " + deepLink);

                    onDownloadState(item, mAdInfo, DEEP_LINK_TRACKER);//发送deepLink激活监播

                    //发送监播
                    onDownloadState(item, mAdInfo, OPEN_TRACKER);
                    //激活完成删除该任务。
                    DownloadDBManager.getInstance(mContext).delete(item);

                    mItems.remove(item);

                    if (item.getListener() != null) {
                        item.getListener().onOpened(item, mAdInfo);
                    }
                    //成功
                    LogUtils.d("DeepLink 2 跳转成功   ：  " + deepLink);
                    return;
                } else {
                    LogUtils.w("DeepLink 2 NotFound  :  " + deepLink);
                }

            } catch (ActivityNotFoundException e) {
                LogUtils.w("DeepLink 2 NotFound  :  " + deepLink);
            } catch (URISyntaxException e) {
                e.printStackTrace();
                LogUtils.w("DeepLink 2 NotFound  :  " + deepLink);
            }
        }

        DownLoadManager.getInstance(mContext).openApp(item, mAdInfo);
    }

    /**
     * 通过包名获取应用程序的名称
     *
     * @param packageName 包名
     * @return 返回包名所对应的应用程序的名称
     */
    private String getProgramNameByPackageName(String packageName) {
        PackageManager pm = mContext.getPackageManager();
        String name = null;
        try {
            name = pm.getApplicationLabel(pm.getApplicationInfo(packageName,
                    PackageManager.GET_META_DATA)).toString();
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        }
        return name;
    }


    /**
     * 打开应用
     *
     * @param packageName    包名
     * @param notificationId 对应该下载任务的通知栏ID
     */
    public boolean openApp(String packageName, int notificationId) {
        try {
            if (TextUtils.isEmpty(packageName)) {
                return false;
            }
            PackageManager packageManager = mContext.getPackageManager();
            Intent intent = packageManager.getLaunchIntentForPackage(packageName);
            if (intent == null) {
                Intent resolveIntent = new Intent(Intent.ACTION_MAIN, null);
                resolveIntent.addCategory(Intent.CATEGORY_LAUNCHER);
                resolveIntent.setPackage(packageName);
                List<ResolveInfo> apps = packageManager.queryIntentActivities(resolveIntent, 0);
                ResolveInfo ri = apps.iterator().next();
                if (ri == null) {
                    return false;
                }

                packageName = ri.activityInfo.packageName;
                String className = ri.activityInfo.name;
                intent = new Intent(Intent.ACTION_MAIN);
                intent.addCategory(Intent.CATEGORY_LAUNCHER);
                ComponentName cn = new ComponentName(packageName, className);
                intent.setComponent(cn);
            }

            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            mContext.startActivity(intent);
            return true;
        } catch (Exception e) {
            if (notificationId != -1) {
                removeNotification(notificationId);
            }
            e.printStackTrace();
            return false;
        }


    }

    /**
     * 启动app
     *
     * @param item 当前下载Item
     */
    private void openApp(DownloadItem item, Ad adInfo) {

        try {
            if (item != null && openApp(item.getPackageName(mContext), item.getDownLoadUrl().hashCode())) {
                //发送监播
                onDownloadState(item, adInfo, OPEN_TRACKER);
                //激活完成删除该任务。
                DownloadDBManager.getInstance(mContext).delete(item);
                mItems.remove(item);

                if (item.getListener() != null) {
                    item.getListener().onOpened(item, adInfo);
                }
            }
            return;
        } catch (Exception e) {
            LogUtils.d("应用启动失败");
            e.printStackTrace();
        }

        LogUtils.w("应用启动失败");
    }

    /**
     * 下载中移除Notification 从库中删除任务。 删除时机 ： 下载中横划，安装完成，过期。
     *
     * @param item 下载任务对象
     */
    public void cancelDownload(DownloadItem item) {
        if (item != null) {
            item.setCurrentState(STATE_CANCEL);
            DownloadDBManager.getInstance(mContext).delete(item);
            mItems.remove(item);
        }
    }


    private void onDownloadState(DownloadItem item, Ad adInfo, String trackerType) {
        if (item != null) {
            Set<String> url = null;
            switch (trackerType) {
                case CLICK_TRACKER:
                    url = adInfo.getTracks(ConstantPool.TrackerType.CLICK);
                    break;
                case DOWNLOAD_START_TRACKER:
                    url = adInfo.getTracks(ConstantPool.TrackerType.S_DOWNLOAD);
                    break;
                case DOWNLOADED_TRACKER:
                    url = adInfo.getTracks(ConstantPool.TrackerType.DOWNLOADED);
                    break;
                case INSTALL_START_TRACKER:
                    url = adInfo.getTracks(ConstantPool.TrackerType.INSTALL_START);
                    break;
                case INSTALLED_TRACKER:
                    url = adInfo.getTracks(ConstantPool.TrackerType.INSTALLED);
                    break;
                case OPEN_TRACKER://打开app
                    url = adInfo.getTracks(ConstantPool.TrackerType.OPEN);
                    break;
                case DEEP_LINK_TRACKER:
                    url = adInfo.getTracks(ConstantPool.TrackerType.DEEP_LINK_OPEN);
                    break;
                default:
                    break;
            }
            if (url != null) {
                try {
                    onDownloadState(url);
                } catch (Exception e) {
                    LogUtils.w("发送监播时发生错误");
                    e.printStackTrace();
                }
                //从库中删除监播，如果监播发送失败，宿主SDK自己做处理。
                DownloadDBManager.getInstance(mContext).deleteTracker(item, trackerType);
            }
        }
    }

    private void onDownloadState(Set<String> url) throws Exception {
        ReportTrackerManager.getInstance().reportTrack(url);
    }

    private void modifyFrom() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2) {
            try {
                Class<?> activityManagerNativeClass = Class.forName("android.app.ActivityManagerNative");
                // 获取 gDefault 这个字段, 想办法替换它
                Field gDefaultField = activityManagerNativeClass.getDeclaredField("gDefault");
                gDefaultField.setAccessible(true);
                Object gDefault = gDefaultField.get(null);
                // 4.x以上的gDefault是一个 android.util.Singleton对象; 我们取出这个单例里面的字段
                Class<?> singleton = Class.forName("android.util.Singleton");
                Field mInstanceField = singleton.getDeclaredField("mInstance");
                mInstanceField.setAccessible(true);
                // ActivityManagerNative 的gDefault对象里面原始的 IActivityManager对象
                Object rawIActivityManager = mInstanceField.get(gDefault);

                // 创建一个这个对象的代理对象, 然后替换这个字段, 让我们的代理对象帮忙干活
                Class<?> iActivityManagerInterface = Class.forName("android.app.IActivityManager");
                Object proxy = Proxy.newProxyInstance(Thread.currentThread().getContextClassLoader(),
                        new Class<?>[]{iActivityManagerInterface}, new IActivityManagerHandler(rawIActivityManager));
                mInstanceField.set(gDefault, proxy);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private class IActivityManagerHandler implements InvocationHandler {
        private Object mBase;

        IActivityManagerHandler(Object object) {
            mBase = object;
        }

        @Override
        public Object invoke(Object o, Method method, Object[] objects) throws Throwable {
            if (method.getName().equals("startActivity")) {
                try {
                    if (objects[1] instanceof String && sIsFromSDK && SDKInfo.SDK.equals(A_SDK)) {
                        objects[1] = "com.android.ddkkddk";
                        sIsFromSDK = false;
                        return method.invoke(mBase, objects);
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            return method.invoke(mBase, objects);
        }


    }
}
