package com.halomobi.download;

import static com.halomobi.common.ConstantPool.EroType.DOWNLOAD_FAILED;
import static com.halomobi.download.bean.DownloadItem.STATE_CANCEL;
import static com.halomobi.download.bean.DownloadItem.STATE_DOWNLOADING;
import static com.halomobi.download.bean.DownloadItem.STATE_FINISH;
import static com.halomobi.download.bean.DownloadItem.STATE_INTERRUPT;
import static com.halomobi.download.bean.DownloadItem.STATE_PAUSE;

import android.text.TextUtils;

import com.halomobi.download.bean.DownloadItem;
import com.halomobi.download.listener.DownloadListener;
import com.halomobi.network.net.AbsNetProcessor;
import com.halomobi.network.net.AbsNetTask;
import com.halomobi.network.net.http.HttpResult;
import com.halomobi.utils.FileUtils;
import com.halomobi.utils.LogUtils;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

public class DownloadProcessor extends AbsNetProcessor {

    private DownloadItem mItem;
    private DownloadListener mListener;

    DownloadProcessor(DownloadItem item) {
        this.mItem = item;
    }

    public void setListener(DownloadListener downloadListener) {
        this.mListener = downloadListener;
    }

    @Override
    protected AbsNetTask createNetTask() {
        return new DownloadTask(mItem);
    }

    /**
     * 需要处理内容
     * 1 外部传入值判空
     * 2 不支持断点续传
     * 3 ETAG发生变化
     */
    @Override
    public void onDataReceived(HttpResult result) throws Exception {
        BufferedInputStream inputStream = null;
        try {
            // 有两个作用1 获取tag和size 2 判断是否需要重新开始任务
            String eTag = "";
            try {
                eTag = result.getHttpResponse().getHeaderField("ETag");
            } catch (Exception e) {
                e.printStackTrace();
            }
            long fileSize = result.getLength();
            //处理eTag
            if (TextUtils.isEmpty(eTag)) {
                eTag = mItem.getDownLoadUrl();
            }
            if (mItem.getCurrentFileSize() == 0) {
                //新任务
                mItem.setCurrentFileSize(fileSize);
                mItem.seteTag(eTag);
            } else {
                //旧任务
                if (!eTag.equals(mItem.geteTag())) {
                    mItem.setCurrentFileSize(0);
                    mItem.setLocalFileSize(0);
                    LogUtils.i("reDownLoad");
                    process();
                    return;
                }
            }

            if (result.getCode() != 206) {//兼容不支持断点续传的情况
                mItem.setLocalFileSize(0);
                mItem.setCurrentFileSize(fileSize);
            }
            String savePath = mItem.getSavePath();
            File saveFile = new File(savePath);
            if (mItem.getLocalFileSize() == 0) {
                if (saveFile.exists()) {
                    if (saveFile.delete()) {
                        LogUtils.d("删除文件成功");
                    } else {
                        LogUtils.e("删除文件失败");
                    }
                }
            }
            onStart();
            mItem.setCurrentState(STATE_DOWNLOADING);
            FileOutputStream accessFile = new FileOutputStream(saveFile, true);
            inputStream = new BufferedInputStream(result.getInputStream());
            byte[] buffer = new byte[2048];
            long updateUiTime = 0;
            int len;
//            accessFile.seek(mItem.getLocalFileSize());
            while ((len = inputStream.read(buffer)) != -1 && mItem.getCurrentState() == STATE_DOWNLOADING) {
                accessFile.write(buffer, 0, len);
                mItem.setLocalFileSize(mItem.getLocalFileSize() + len);
                if (System.currentTimeMillis() - updateUiTime > 1000) {
                    onProcessing();
                    updateUiTime = System.currentTimeMillis();
                }
            }
            int state = mItem.getCurrentState();
            if (state == STATE_PAUSE || state == STATE_INTERRUPT || state == STATE_CANCEL) {
                // 用户行为
                onStop();
            } else if (mItem.getCurrentFileSize() != 0 && mItem.getCurrentFileSize() == mItem.getLocalFileSize()) {
                // 正常完成
                onFinish();
            } else {
                //下载出错
                onError(DOWNLOAD_FAILED, "File Size Cannot Match !");
            }
        } catch (IOException e) {
            onError(DOWNLOAD_FAILED, "下载失败 : " + LogUtils.getThrowable(e));
            e.printStackTrace();
        } finally {
            FileUtils.close(inputStream);
        }
    }

    @Override
    public void onErrorReceived(String eroMsg, int eroCode) {
        LogUtils.e("----- onErrorReceived -----");
        LogUtils.e("----- eroCode : " + eroCode);
        LogUtils.e("----- eroMsg : " + eroMsg);

        onError(DOWNLOAD_FAILED, "下载失败 : eroCode : " + eroCode + " , eroMsg : " + eroMsg);

    }

    @Override
    public void onNetworkError() {
        LogUtils.e("----- onNetworkError -----");
    }


    private void onFinish() {
        mItem.setCurrentState(STATE_FINISH);
        if (mListener != null) {
            mListener.onFinish(mItem);
        }
    }

    private void onProcessing() {
        if (mListener != null) {
            mListener.onProcessing(mItem);
        }
    }

    private void onStart() {

        if (mListener != null) {
            mListener.onStart(mItem);
        }
    }

    private void onStop() {
        if (mListener != null) {
            mListener.onStop(mItem);
        }
    }

    private void onError(int errorCode, String errorMsg) {
//        Utils.showToastSafe("下载失败，请重试");
        LogUtils.e("下载失败:" +  errorMsg + "\n DownloadUrl : " + mItem.getDownLoadUrl());
        mItem.setCurrentState(STATE_INTERRUPT);
        if (mListener != null) {
            mListener.onError(mItem, errorCode, errorMsg + "\n DownloadUrl : " + mItem.getDownLoadUrl());
        }
    }

}
