package com.halomobi.download.bean;

import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.text.TextUtils;
import androidx.core.app.NotificationCompat;

import com.halomobi.download.listener.DownloadStateListener;
import com.halomobi.utils.NetworkUtils;


public class DownloadItem {
    public static final int STATE_DOWNLOADING = 1;
    public static final int STATE_WAITING = 2;
    public static final int STATE_PAUSE = 3;
    public static final int STATE_INTERRUPT = 4;
    public static final int STATE_FINISH = 5;
    public static final int STATE_INSTALLED = 6;
    public static final int STATE_CANCEL = 7;

    private String downLoadUrl = "";
    private String description;
    private String savePath;
    private String downloadStartTracker;
    private String downloadedTracker;
    private String installStartTracker;
    private String installedTracker;
    private String openTracker;
    private String name;
    private Bitmap icon;
    private long expirationTime = 86400 * 1000;
    private String packageName;
    private String clickTracker;
    private String deepLinkTracker;
    private NotificationCompat.Builder builder;
    private String deepLink;
    private long localFileSize;

    private long currentFileSize;
    private int currentState;
    private int netType = NetworkUtils.NET_WIFI;
    private DownloadStateListener listener;

    private String eTag;

    public String getDeepLinkTracker() {
        return deepLinkTracker;
    }

    public void setDeepLinkTracker(String deepLinkTracker) {
        this.deepLinkTracker = deepLinkTracker;
    }

    public void setDeepLink(String deepLink) {
        this.deepLink = deepLink;
    }

    public String getDeepLink() {
        return deepLink;
    }

    public DownloadStateListener getListener() {
        return listener;
    }

    public void setListener(DownloadStateListener listener) {
        this.listener = listener;
    }

    public int getNetType() {
        return netType;
    }

    public void setNetType(int netType) {
        this.netType = netType;
    }

    public String getPackageName(Context context) {

        if (TextUtils.isEmpty(packageName)) {
            setPackageName(getPackageName4File(context, savePath));
        }
        return packageName;
    }

    public void setIcon(Bitmap icon) {
        this.icon = icon;
    }

    public Bitmap getIcon(){
        return icon;
    }

    public void setPackageName(String packageName) {
        this.packageName = packageName;
    }

    public long getExpirationTime() {

        return expirationTime;
    }

    public void setExpirationTime(long expirationTime) {
        this.expirationTime = expirationTime;
    }

    public String getDownloadStartTracker() {
        return downloadStartTracker;
    }

    public void setDownloadStartTrackerTracker(String downloadStartTracker) {
        this.downloadStartTracker = downloadStartTracker;
    }

    public String getDownloadedTracker() {
        return downloadedTracker;
    }

    public void setDownloadedTracker(String downloadedTracker) {
        this.downloadedTracker = downloadedTracker;
    }

    public String getInstallStartTracker() {
        return installStartTracker;
    }

    public void setInstallStartTracker(String installStartTracker) {
        this.installStartTracker = installStartTracker;
    }

    public String getInstalledTracker() {
        return installedTracker;
    }

    public void setInstalledTracker(String installedTracker) {
        this.installedTracker = installedTracker;
    }

    public String getOpenTracker() {
        return openTracker;
    }

    public void setOpenTracker(String openTracker) {
        this.openTracker = openTracker;
    }

    public String getClickTracker() {
        return clickTracker;
    }

    public void setClickTracker(String clickTracker) {
        this.clickTracker = clickTracker;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public NotificationCompat.Builder getBuilder() {
        return builder;
    }

    public void setBuilder(NotificationCompat.Builder builder) {
        this.builder = builder;
    }

    public String getDownLoadUrl() {
        return downLoadUrl;
    }

    public void setDownLoadUrl(String downLoadUrl) {
        this.downLoadUrl = downLoadUrl;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getSavePath() {
        return savePath;
    }

    public void setSavePath(String savePath) {
        this.savePath = savePath;
    }

    public long getLocalFileSize() {
        return localFileSize;
    }

    public void setLocalFileSize(long localFileSize) {
        this.localFileSize = localFileSize;
    }

    public int getCurrentState() {
        return currentState;
    }

    public void setCurrentState(int currentState) {
        this.currentState = currentState;
    }

    public long getCurrentFileSize() {
        return currentFileSize;
    }

    public void setCurrentFileSize(long currentFileSize) {
        this.currentFileSize = currentFileSize;
    }

    public String geteTag() {
        return eTag;
    }

    public void seteTag(String eTag) {
        this.eTag = eTag;
    }

    private String getPackageName4File(Context context, String path) {
        PackageManager pm = context.getPackageManager();
        PackageInfo info = pm.getPackageArchiveInfo(path,
                PackageManager.GET_ACTIVITIES);
        if (info != null) {
            ApplicationInfo appInfo = info.applicationInfo;
            return appInfo.packageName;
        }
        return "";
    }

    @Override
    public String toString() {
        return "DownloadItem{" +
                "downLoadUrl='" + downLoadUrl + '\'' +
                ", description='" + description + '\'' +
                ", savePath='" + savePath + '\'' +
                ", downloadedTracker='" + downloadedTracker + '\'' +
                ", installedTracker='" + installedTracker + '\'' +
                ", openTracker='" + openTracker + '\'' +
                ", name='" + name + '\'' +
                ", clickTracker='" + clickTracker + '\'' +
                ", expirationTime=" + expirationTime +
                ", packageName='" + packageName + '\'' +
                ", builder=" + builder +
                ", localFileSize=" + localFileSize +
                ", currentFileSize=" + currentFileSize +
                ", currentState=" + currentState +
                ", eTag='" + eTag + '\'' +
                '}';
    }

    @Override
    public boolean equals(Object o) {
        if (getDownLoadUrl() == null || !(o instanceof DownloadItem)) {
            return false;
        }
        String downloadUrl = ((DownloadItem) o).getDownLoadUrl();
        return getDownLoadUrl().contentEquals(downloadUrl);
    }

    @Override
    public int hashCode() {
        return super.hashCode();
    }
}