package com.halomobi.network.request.ad.bean;

import static com.halomobi.common.ConstantPool.UA_KEY;
import static com.halomobi.network.request.host.RequestIpManager.IP_ADDRESS;

import android.text.TextUtils;
import android.util.Log;

import com.halomobi.common.Config;
import com.halomobi.common.ConstantPool;
import com.halomobi.sdk.BuildConfig;
import com.halomobi.utils.KeyUtil;
import com.halomobi.utils.NetworkUtils;
import com.halomobi.utils.SystemUtils;
import com.halomobi.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;

public class AdRequest {
    private String configStr;
    /**
     * 应用信息
     */
    private App app;
    /**
     * 设备信息
     */
    private HlDevice device;
    /**
     * 广告位信息
     */
    private AdSlot adSlot;

    private AdRequest(AdParameter p) {
        app = new App(p.appId, p.pk);
        device = new HlDevice(p);
        adSlot = new AdSlot(p.adSlotId, p.adType.getType());
    }

    public static byte[] getParams(AdParameter parameter) {
        AdRequest jParam = new AdRequest(parameter);
        return serializeJson(jParam, parameter);
    }

    private static byte[] serializeJson(AdRequest parameter, AdParameter currentParameter) {
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("id",Config.getUUID());
            jsonObject.put("at",1);
            jsonObject.put("tmax",5000);
            String[] cur = new String[]{"USD"};
            jsonObject.put("cur", new JSONArray(Arrays.asList(cur)));
            HlDevice device = parameter.device;
            AdSlot adSlot = parameter.adSlot;
            if (adSlot != null) {
                JSONArray impArray = new JSONArray();
                JSONObject impObject = new JSONObject();
                impObject.put("id","1");
                impObject.put("bidfloor", SystemUtils.getInfoIntByJson("bidfloor"));
                impObject.put("bidfloorcur","USD");
                impObject.put("secure", 1);
                impObject.put("tagid", adSlot.slot_id);

                JSONObject adObject = new JSONObject();
//                if (device != null) {
//                    adObject.put("w",device.w);
//                    adObject.put("h",device.h);
//                }
//                adObject.put("hmin",0);
//                adObject.put("wmin",0);
//                adObject.put("pos",0);
//                adObject.put("topframe",0);

                int slotType = adSlot.slot_type;
                if (slotType == ConstantPool.AdType.BANNER.getType()) {
                    if (currentParameter.adViewSize.length == 2) {
                      adObject.put("w", currentParameter.adViewSize[0]);
                      adObject.put("h", currentParameter.adViewSize[1]);
                    }
                    impObject.put("banner",adObject);
                } else if (slotType == ConstantPool.AdType.INSERT.getType()) {
                    if (currentParameter.adViewSize.length == 2) {
                      adObject.put("w", currentParameter.adViewSize[0]);
                      adObject.put("h", currentParameter.adViewSize[1]);
                    }
                    impObject.put("banner",adObject);
                    impObject.put("instl", 1);
                } else if (slotType == ConstantPool.AdType.REWARDVIDEO.getType()) {
                    if (currentParameter.adViewSize.length == 2) {
                       adObject.put("w", currentParameter.adViewSize[0]);
                       adObject.put("h", currentParameter.adViewSize[1]);
                    }
                    impObject.put("video",adObject);
                } else if (slotType == ConstantPool.AdType.NATIVE.getType()) {
                    JSONObject requestObj = new JSONObject();
                    JSONArray assetsArr = new JSONArray();
                    JSONObject assetsSubObj = new JSONObject();
                    assetsSubObj.put("id", 1);
                    assetsSubObj.put("required", 1);
                    JSONObject assetsImgObj = new JSONObject();
                    assetsImgObj.put("type", 3);
                    if (currentParameter.adViewSize.length == 2) {
                       assetsImgObj.put("w", currentParameter.adViewSize[0]);
                       assetsImgObj.put("h", currentParameter.adViewSize[1]);
                       assetsImgObj.put("wmin", currentParameter.adViewSize[0]);
                       assetsImgObj.put("hmin", currentParameter.adViewSize[1]);
                    }
//                    assetsImgObj.put("w", Utils.getScreenSize(Utils.getContext(),true)[0]);
//                    assetsImgObj.put("h", 450);
//                    JSONArray mimesImgArr = new JSONArray();
                    List<String> imgList = new ArrayList<>();
                    imgList.add("image/jpg");
                    imgList.add("image/png");
                    JSONArray mimesImgArr = new JSONArray(imgList);
//                    mimesImgArr.put("image/jpg");
//                    mimesImgArr.put("image/png");
                    assetsImgObj.put("mimes", mimesImgArr);
                    assetsSubObj.put("img", assetsImgObj);
                    assetsArr.put(assetsSubObj);
                    requestObj.put("ver", "1.0");
                    requestObj.put("assets", assetsArr);
                    JSONObject subNativeObj = new JSONObject();
                    subNativeObj.put("native", requestObj);
//                    String requestStr = "{\"assets\":[{\"id\":1,\"required\":1,\"img\":{\"type\":3,\"w\":720,\"h\":1280,\"mimes\":[\"image/jpg\",\"image/png\"]}}],\"ver\":\"1.0\"}";
                    adObject.put("request", subNativeObj.toString());
                    impObject.put("native",adObject);
                }
                impArray.put(impObject);
                jsonObject.put("imp", impArray);
            }

            App app = parameter.app;
            if (app != null) {
                JSONObject appObject = new JSONObject();
                appObject.put("bundle",app.package_name);
                appObject.put("id",app.app_id);
                appObject.put("name",app.app_name);
                appObject.put("ver",app.app_version);
//                appObject.put("storeurl","https://play.google.com/store/apps/details?id=com.block.juggle");//待定

                JSONObject pubObject = new JSONObject();
                pubObject.put("id","113");//待定

                appObject.put("publisher", pubObject);

                jsonObject.put("app",appObject);
            }

            if (device != null) {
                JSONObject deviceObject = new JSONObject();
                deviceObject.put("carrier", String.valueOf(device.carrier));
                deviceObject.put("connectiontype",device.connect_type);
                deviceObject.put("devicetype",device.device_type);
                deviceObject.put("w",device.w);
                deviceObject.put("h",device.h);
                deviceObject.put("ifa",device.ifa);
                deviceObject.put("ip",device.ip);
                deviceObject.put("make",device.make);
                deviceObject.put("model",device.model);
                deviceObject.put("os","android");
                deviceObject.put("osv",device.os_version);
                deviceObject.put("language", device.language);
                deviceObject.put("ua",device.userAgent);

                JSONObject geoObject = new JSONObject();
                geoObject.put("lat",device.latitude);
                geoObject.put("lon",device.longitude);
                geoObject.put("city",device.city);
                geoObject.put("country",device.country);
                geoObject.put("region",device.city);
                geoObject.put("type",2);
                deviceObject.put("geo", geoObject);

                jsonObject.put("device", deviceObject);
            }

            JSONObject userObject = new JSONObject();
            JSONObject extUserObject = new JSONObject();
            userObject.put("ext", extUserObject);
            jsonObject.put("user", userObject);
            if (BuildConfig.DEBUG) {
                jsonObject.put("test", 1);
            }
//            Log.e("11111","===js===" + jsonObject);
        } catch (JSONException e) {
            throw new RuntimeException(e);
        }
        try {
//            return KeyUtil.encryptCurrentAES(SystemUtils.formatCurrentData(), jsonObject.toString());
            return jsonObject.toString().getBytes();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    private static class App {

        private String app_id;
        private String app_version;
        private String app_name;
        private String package_name;

        App(String app_id, String pk) {
            if (TextUtils.isEmpty(app_id)) {
                this.app_id = Config.mAppId;
            } else {
                this.app_id = app_id;
            }
            if (TextUtils.isEmpty(pk)) {
                package_name = Config.getAppPackageName();
            } else {
                package_name = pk;
            }
            app_name = Config.getAppName(package_name);
            app_version = Config.getAppVersion();

        }
    }

    public static class HlDevice {
        private String userAgent = Utils.getSp().getString(UA_KEY, "");
        private String ip = Utils.getSp().getString(IP_ADDRESS, "");
        private String imei = SystemUtils.getInfoByJson("did");
        private String oaId = SystemUtils.getInfoByJson("oaId");
        private String androidID = SystemUtils.getInfoByJson("dpid");
        private String mac = SystemUtils.getInfoByJson("mac");
        private int carrier = SystemUtils.getOperatorsName();
        private String systemUpdate = SystemUtils.getUpdate();
        private String systemBoot = SystemUtils.getBoot();
        private double latitude = SystemUtils.getLatitude();
        private double longitude = SystemUtils.getLongitude();
        private String city = SystemUtils.getCity();
        private String storeVersion = SystemUtils.getAppStoreVersion();
        private String make = Config.getVendor();
        private String model = Config.getDeviceModel();
        private int os_type = 2;
        private String os_version = Config.getOSVersion();
        private int w;
        private int h;
        private int connect_type = Math.max(0, NetworkUtils.getNetworkType());
        private int device_type = Config.getDeviceType();
        private String ifa = SystemUtils.getInfoByJson("ifa");
        private String language = Locale.getDefault().getLanguage();
        private String country = SystemUtils.getCurrentCountry();
        public HlDevice(AdParameter parameter) {
            w = Utils.getScreenSize(Utils.getContext(),true)[0];
            h = Utils.getScreenSize(Utils.getContext(),true)[1];
        }
    }

    private static class AdSlot {

        private String app_id;
        private String slot_id;
        private int slot_type;

        AdSlot(String adSlotId, int adSlotType) {
            this.app_id = Config.mAppId;
            this.slot_id = adSlotId;
            this.slot_type = adSlotType;
        }
    }

}
