/*
 * File Name: LogUtils.java 
 * History:
 * Created by mwqi on 2014-4-4
 */
package com.halomobi.utils;

import android.util.Log;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;


public class LogUtils {

    /**
     * 日志输出级别NONE
     */
    public static final int LEVEL_NONE = 0;
    /**
     * 日志输出级别V
     */
    public static final int LEVEL_VERBOSE = 1;
    /**
     * 日志输出级别D
     */
    public static final int LEVEL_DEBUG = 2;
    /**
     * 日志输出级别I
     */
    public static final int LEVEL_INFO = 3;
    /**
     * 日志输出级别W
     */
    public static final int LEVEL_WARN = 4;
    /**
     * 日志输出级别E
     */
    public static final int LEVEL_ERROR = 5;
    private static final String TEMPLATE1 = "%s(L:%d)  %s";
    /**
     * 日志输出时的TAG
     */
    private static String mTag = SDKInfo.TAG;
    /**
     * 是否允许输出log
     */
    private static int mDebuggable = LEVEL_NONE;

    public static void setDeBugModel(int debug) {
//        if (Utils.getClipBoard()) {
//            mDebuggable = LEVEL_ERROR;
//        } else {
            mDebuggable = debug;
//        }
    }

    public static String getThrowable(Throwable e) {
        StringBuffer sb_ero = new StringBuffer("\n----------------------------------");
        sb_ero.append("threadId:" + Thread.currentThread().getId() + "\n");
        sb_ero.append("error:" + e.getMessage() + "\n");

        StackTraceElement[] elements = e.getStackTrace();

        for (StackTraceElement element : elements) {

            sb_ero.append(element.toString() + "\n");

        }
        sb_ero.append("\n" + "causeBy:" + e.getMessage() + "\n");
        Throwable cause = e.getCause();
        if (cause != null) {
            StackTraceElement[] causesElements = cause.getStackTrace();

            for (StackTraceElement element : causesElements) {

                sb_ero.append(element.toString() + "\n");

            }
        }
        sb_ero.append("\n----------------------------------------");
        return sb_ero.toString();
    }

//==================================================================================

    /**
     * 以级别为 v 的形式输出LOG
     */
    public static void v(String log) {
        if (mDebuggable >= LEVEL_VERBOSE) {

            StackTraceElement[] sElements = new Throwable().getStackTrace();

            Log.v(sElements[1].getClassName(), createLog(sElements, log));
        }
    }

    /**
     * 以级别为 d 的形式输出LOG
     */
    public static void d(String log) {
        if (mDebuggable >= LEVEL_VERBOSE) {
            StackTraceElement[] sElements = new Throwable().getStackTrace();

            Log.d(sElements[1].getClassName(), createLog(sElements, log));
        }
    }

    /**
     * 以级别为 i 的形式输出LOG
     */
    public static void i(String log) {
        if (mDebuggable >= LEVEL_VERBOSE) {
            StackTraceElement[] sElements = new Throwable().getStackTrace();

            Log.i(sElements[1].getClassName(), createLog(sElements, log));
        }
    }

    /**
     * 以级别为 w 的形式输出LOG
     */
    public static void w(String log) {
        if (mDebuggable >= LEVEL_VERBOSE) {
            StackTraceElement[] sElements = new Throwable().getStackTrace();

            Log.w(sElements[1].getClassName(), createLog(sElements, log));
        }
    }

    /**
     * 以级别为 w 的形式输出LOG
     */
    public static void w(Throwable throwable) {
        if (mDebuggable >= LEVEL_VERBOSE) {
            String log = createThrowableInfo(throwable);
            StackTraceElement[] sElements = new Throwable().getStackTrace();

            Log.w(sElements[1].getClassName(), createLog(sElements, log));
        }
    }

    /**
     * 以级别为 e 的形式输出LOG
     */
    public static void e(String log) {
        if (mDebuggable >= LEVEL_VERBOSE) {
            StackTraceElement[] sElements = new Throwable().getStackTrace();

            Log.e(sElements[1].getClassName(), createLog(sElements, log));
        }
    }

    /**
     * 以级别为 e 的形式输出LOG
     */
    public static void e(Throwable throwable) {
        if (mDebuggable >= LEVEL_VERBOSE) {
            String log = createThrowableInfo(throwable);
            StackTraceElement[] sElements = new Throwable().getStackTrace();

            Log.e(sElements[1].getClassName(), createLog(sElements, log));
        }
    }

    /**
     * 拼接log
     *
     * @param sElements
     * @param log
     * @return
     */
    private static synchronized String createLog(StackTraceElement[] sElements, String log) {
        String methodName = sElements[1].getMethodName();
        int lineNumber = sElements[1].getLineNumber();
        String logInfo = String.format(TEMPLATE1, methodName, lineNumber, log);
        // 存储log2file
        // saveLog(saveLevel, logInfo);
        return logInfo;
    }

    private static String createThrowableInfo(Throwable ex) {

        StringBuffer sb_record = new StringBuffer();
        StackTraceElement[] elements = ex.getStackTrace();
        for (StackTraceElement element : elements) {

            sb_record.append(element.toString() + "\n");

        }
        sb_record.append("\n" + "causeBy:" + ex.getMessage() + "\n");
        Throwable cause = ex.getCause();
        if (cause != null) {
            StackTraceElement[] causesElements = cause.getStackTrace();

            for (StackTraceElement element : causesElements) {

                sb_record.append(element.toString() + "\n");

            }
        }
        return sb_record.toString();
    }

    /**
     * 转化成可读时间
     *
     * @param time 时间戳
     * @return 可读时间
     */
    public static String formatDate(long time) {
        return new SimpleDateFormat("yy-MM-dd hh:mm:ss.SSS", Locale.getDefault()).format(new Date(time));
    }
}
