package com.halomobi.utils;


import static com.halomobi.utils.Utils.getContext;

import android.Manifest;
import android.annotation.TargetApi;
import android.app.Activity;
import android.content.pm.PackageManager;
import android.os.Build;
import java.util.ArrayList;
import java.util.List;


public class RequestPermissionManager {

    public static final int DEFAULT_REQUEST_CODE = 10001;

    private static RequestPermissionManager mInstance;

    private RequestPermissionManager() {
    }

    public synchronized static RequestPermissionManager getInstance() {
        return mInstance == null ? mInstance = new RequestPermissionManager() : mInstance;
    }

    public boolean shouldShowRequestPermissionRationale(Activity activity, String permission) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M && activity != null) {
            return activity.shouldShowRequestPermissionRationale(permission);
        }
        return false;
    }

    /**
     * 初始化时调用，请求SDK所需的所有权限
     * 位置、手机硬件信息、SD卡权限等
     */
    public void requestPermissions(Activity activity, int requestCode) {
        if (activity == null) {
            LogUtils.w(" activity == null");
            return;
        }
        List<String> permissions = new ArrayList<>();

        //获取手机信息
        if (!checkSelfPermission(Manifest.permission.READ_PHONE_STATE)) {
            permissions.add(Manifest.permission.READ_PHONE_STATE);
        }
        //获取位置
//        if (!checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION)) {
//            permissions.add(Manifest.permission.ACCESS_FINE_LOCATION);
//        }
        //外部存储
        if (!checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
            permissions.add(Manifest.permission.WRITE_EXTERNAL_STORAGE);
        }
        requestPermissions(activity, permissions.toArray(new String[permissions.size()]), requestCode);
    }

    private void requestPermissions(Activity activity, String[] permission, int requestCode) {
        if (permission == null || permission.length < 1) {
            LogUtils.d("没有权限需要申请");
            return;
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            activity.requestPermissions(permission, requestCode);
        }
    }

    /**
     * 检测权限
     * Android 6.0之前版本默认返回true
     *
     * @param permission 权限名称
     * @return 是否具有该权限
     */
    public static boolean checkSelfPermission(String permission) {
        return Build.VERSION.SDK_INT < Build.VERSION_CODES.M
                || PackageManager.PERMISSION_GRANTED == getContext().checkSelfPermission(permission);
    }
}
