package com.halomobi.utils;

import static com.halomobi.common.ConstantPool.TrackerType.CLICK;
import static com.halomobi.common.ConstantPool.TrackerType.DOWNLOADED;
import static com.halomobi.common.ConstantPool.TrackerType.INSTALLED;
import static com.halomobi.common.ConstantPool.TrackerType.INSTALL_START;
import static com.halomobi.common.ConstantPool.TrackerType.OPEN;
import static com.halomobi.common.ConstantPool.TrackerType.S_DOWNLOAD;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Bitmap;
import android.os.Build;
import android.text.TextUtils;
import android.webkit.DownloadListener;
import android.webkit.WebChromeClient;
import android.webkit.WebSettings;
import android.webkit.WebView;
import android.webkit.WebViewClient;

import com.halomobi.common.ConstantPool;
import com.halomobi.download.DownLoadManager;
import com.halomobi.download.bean.AppInfo;
import com.halomobi.download.bean.DownloadItem;
import com.halomobi.download.listener.DownloadStateListener;
import com.halomobi.network.listeners.TrackerTaskListener;
import com.halomobi.network.report.track.ReportTrackerManager;
import com.halomobi.network.request.ad.bean.Ad;

import java.util.Arrays;
import java.util.HashSet;

/**
 * @author Feng Zhanao
 * @date 2019/3/14
 * @description
 */
public class WebViewUtils {
    private HashSet<String> dUrls = new HashSet<>();
    private Context mContext;
    private Ad mAd;

    @SuppressLint("SetJavaScriptEnabled")
    public void initWebView(Context context, WebView wView, final Ad ad) {
        mContext = context;
        mAd = ad;
        WebSettings webSettings = wView.getSettings();
        webSettings.setMixedContentMode(WebSettings.MIXED_CONTENT_ALWAYS_ALLOW);
        webSettings.setSupportZoom(false);
        webSettings.setJavaScriptEnabled(true);
        webSettings.setJavaScriptCanOpenWindowsAutomatically(true);
        webSettings.setUseWideViewPort(false);// 这个很关键
        webSettings.setLoadWithOverviewMode(true);
        webSettings.setBlockNetworkImage(false);
        WebView.setWebContentsDebuggingEnabled(true);
        webSettings.setDomStorageEnabled(true);
        webSettings.setAllowFileAccess(true);
        // 取消滚动条
        wView.setHorizontalScrollBarEnabled(false);//水平不显示
        wView.setVerticalScrollBarEnabled(false); //垂直不显示
        wView.setDownloadListener(new DownloadListener() {
            AppInfo appInfo;

            @Override
            public void onDownloadStart(final String url, String userAgent, String contentDisposition, String mimetype, long contentLength) {
                LogUtils.e("fza" + "downloadUrl:" + url);
                if (TextUtils.isEmpty(url) || contentLength <= 0) {
                    return;
                }

                if (dUrls.contains(url)) {
//                    Utils.showToastSafe("正在下载中...");
                    return;
                }

                if (appInfo != null && url.equals(appInfo.getDownloadUrl())) {
                    String packageName = appInfo.getPackageName();
                    if (DownLoadManager.getInstance(mContext).isAppInstalled(packageName)) {//检测是否已经安装
                        if (DownLoadManager.getInstance(mContext).openApp(packageName, -1)) {
                            LogUtils.i("WebView 打开 已安装 app--->" + packageName);
                        }

                    } else {
                        dUrls.add(url);

                        DownLoadManager.getInstance(mContext).downloadWithShow(createDownloadItem(url, mAd), mAd);

                        appInfo = new AppInfo(url, contentLength);
                    }
                } else {

                    dUrls.add(url);

                    DownLoadManager.getInstance(mContext).downloadWithShow(createDownloadItem(url, mAd), mAd);

                    appInfo = new AppInfo(url, contentLength);

                }
            }
        });
    }


    private DownloadItem createDownloadItem(final String url, Ad mAd) {
        DownloadItem item = new DownloadItem();
        item.setName(KeyUtil.getMD5(url));//设置下载项目名称
        item.setDownLoadUrl(url.trim());//下载地址
        if (mAd.ctop == 4) {
            //替换点击位置
            item.setClickTracker(Arrays.toString(mAd.getTracks(CLICK).toArray()));
        }
        item.setDownloadStartTrackerTracker(Arrays.toString(mAd.getTracks(S_DOWNLOAD).toArray()));
        item.setDownloadedTracker(Arrays.toString(mAd.getTracks(DOWNLOADED).toArray()));
        item.setInstallStartTracker(Arrays.toString(mAd.getTracks(INSTALL_START).toArray()));
        item.setInstalledTracker(Arrays.toString(mAd.getTracks(INSTALLED).toArray()));
        item.setOpenTracker(Arrays.toString(mAd.getTracks(OPEN).toArray()));
        item.setListener(new DownloadStateListener() {

            @Override
            public void onDownloadError(DownloadItem item, int errorCode, String errorMsg) {
                LogUtils.e("fza" + "onDownloadError:");
                dUrls.remove(url);
            }

            @Override
            public void onDownloadStart(DownloadItem item, Ad adInfo) {
                LogUtils.e("fza" + "onDownloadStart:");

            }

            @Override
            public void onDownloaded(DownloadItem item, Ad adInfo) {
                LogUtils.e("fza" + "onDownloaded:" + url);
                dUrls.remove(url);
            }

            @Override
            public void onInstalled(DownloadItem item, Ad adInfo) {
                LogUtils.e("fza" + "onInstalled:" + url);

            }

            @Override
            public void onOpened(DownloadItem item, Ad adInfo) {
                LogUtils.e("fza" + "onOpened:" + url);
            }

        });

        return item;
    }

}
